@@ -38,9 +38,19 @@ push(char *bfr, int size, MMIOT *f)
 }
 
 
+/*
+ * push a character into the generator input buffer
+ */
+static void 
+pushc(char c, MMIOT *f)
+{
+    EXPAND(f->in) = c;
+}
+
+
 /* look <i> characters ahead of the cursor.
  */
-static int
+static inline int
 peek(MMIOT *f, int i)
 {
 
@@ -52,7 +62,7 @@ peek(MMIOT *f, int i)
 
 /* pull a byte from the input buffer
  */
-static int
+static inline int
 pull(MMIOT *f)
 {
     return ( f->isp < S(f->in) ) ? T(f->in)[f->isp++] : EOF;
@@ -61,23 +71,27 @@ pull(MMIOT *f)
 
 /* return a pointer to the current position in the input buffer.
  */
-static char*
+static inline char*
 cursor(MMIOT *f)
 {
     return T(f->in) + f->isp;
 }
 
 
-static int
+static inline int
 isthisspace(MMIOT *f, int i)
 {
     int c = peek(f, i);
 
-    return isspace(c) || (c == EOF);
+    if ( c == EOF )
+	return 1;
+    if ( c & 0x80 )
+	return 0;
+    return isspace(c) || (c < ' ');
 }
 
 
-static int
+static inline int
 isthisalnum(MMIOT *f, int i)
 {
     int c = peek(f, i);
@@ -86,7 +100,7 @@ isthisalnum(MMIOT *f, int i)
 }
 
 
-static int
+static inline int
 isthisnonword(MMIOT *f, int i)
 {
     return isthisspace(f, i) || ispunct(peek(f,i));
@@ -183,17 +197,27 @@ Qem(MMIOT *f, char c, int count)
 /* generate html from a markup fragment
  */
 void
-___mkd_reparse(char *bfr, int size, int flags, MMIOT *f)
+___mkd_reparse(char *bfr, int size, int flags, MMIOT *f, char *esc)
 {
     MMIOT sub;
+    struct escaped e;
 
     ___mkd_initmmiot(&sub, f->footnotes);
     
     sub.flags = f->flags | flags;
     sub.cb = f->cb;
+    sub.ref_prefix = f->ref_prefix;
+
+    if ( esc ) {
+	sub.esc = &e;
+	e.up = f->esc;
+	e.text = esc;
+    }
+    else
+	sub.esc = f->esc;
 
     push(bfr, size, &sub);
-    EXPAND(sub.in) = 0;
+    pushc(0, &sub);
     S(sub.in)--;
     
     text(&sub);
@@ -205,6 +229,23 @@ ___mkd_reparse(char *bfr, int size, int flags, MMIOT *f)
 }
 
 
+/*
+ * check the escape list for special cases
+ */
+static int
+escaped(MMIOT *f, char c)
+{
+    struct escaped *thing = f->esc;
+
+    while ( thing ) {
+	if ( strchr(thing->text, c) )
+	    return 1;
+	thing = thing->up;
+    }
+    return 0;
+}
+
+
 /*
  * write out a url, escaping problematic characters
  */
@@ -231,7 +272,7 @@ puturl(char *s, int size, MMIOT *f, int display)
 	    Qstring("%22", f);
 	else if ( isalnum(c) || ispunct(c) || (display && isspace(c)) )
 	    Qchar(c, f);
-	else if ( c == 003 )	/* untokenize ^C */
+	else if ( c == MKD_EOLN )	/* untokenize hard return */
 	    Qstring("  ", f);
 	else
 	    Qprintf(f, "%%%02X", c);
@@ -351,6 +392,53 @@ linkysize(MMIOT *f, Footnote *ref)
 }
 
 
+/* extract a <...>-encased url from the input stream.
+ * (markdown 1.0.2b8 compatibility; older versions
+ * of markdown treated the < and > as syntactic
+ * sugar that didn't have to be there.  1.0.2b8 
+ * requires a closing >, and then falls into the
+ * title or closing )
+ */
+static int
+linkybroket(MMIOT *f, int image, Footnote *p)
+{
+    int c;
+    int good = 0;
+
+    T(p->link) = cursor(f);
+    for ( S(p->link)=0; (c = pull(f)) != '>'; ++S(p->link) ) {
+	/* pull in all input until a '>' is found, or die trying.
+	 */
+	if ( c == EOF )
+	    return 0;
+	else if ( (c == '\\') && ispunct(peek(f,2)) ) {
+	    ++S(p->link);
+	    pull(f);
+	}
+    }
+
+    c = eatspace(f);
+
+    /* next nonspace needs to be a title, a size, or )
+     */
+    if ( ( c == '\'' || c == '"' ) && linkytitle(f,c,p) )
+	good=1;
+    else if ( image && (c == '=') && linkysize(f,p) )
+	good=1;
+    else 
+	good=( c == ')' );
+
+    if ( good ) {
+	if ( peek(f, 1) == ')' )
+	    pull(f);
+	    
+	___mkd_tidy(&p->link);
+    }
+
+    return good;
+} /* linkybroket */
+
+
 /* extract a (-prefixed url from the input stream.
  * the label is either of the format `<link>`, where I
  * extract until I find a >, or it is of the format
@@ -368,6 +456,8 @@ linkyurl(MMIOT *f, int image, Footnote *p)
 
     if ( c == '<' ) {
 	pull(f);
+	if ( !(f->flags & MKD_1_COMPAT) )
+	    return linkybroket(f,image,p);
 	mayneedtotrim=1;
     }
 
@@ -405,10 +495,10 @@ static struct _protocol {
     int   nlen;
 } protocol[] = { 
 #define _aprotocol(x)	{ x, (sizeof x)-1 }
-    _aprotocol( "https://" ), 
-    _aprotocol( "http://" ), 
-    _aprotocol( "news://" ),
-    _aprotocol( "ftp://" ), 
+    _aprotocol( "https:" ), 
+    _aprotocol( "http:" ), 
+    _aprotocol( "news:" ),
+    _aprotocol( "ftp:" ), 
 #undef _aprotocol
 };
 #define NRPROTOCOLS	(sizeof protocol / sizeof protocol[0])
@@ -445,9 +535,9 @@ typedef struct linkytype {
 } linkytype;
 
 static linkytype imaget = { 0, 0, "<img src=\"", "\"",
-			     1, " alt=\"", "\" />", DENY_IMG|INSIDE_TAG, IS_URL };
+			     1, " alt=\"", "\" />", MKD_NOIMAGE|MKD_TAGTEXT, IS_URL };
 static linkytype linkt  = { 0, 0, "<a href=\"", "\"",
-                             0, ">", "</a>", DENY_A, IS_URL };
+                             0, ">", "</a>", MKD_NOLINKS, IS_URL };
 
 /*
  * pseudo-protocols for [][];
@@ -457,8 +547,8 @@ static linkytype linkt  = { 0, 0, "<a href=\"", "\"",
  * raw: just dump the link without any processing
  */
 static linkytype specials[] = {
-    { "id:", 3, "<a id=\"", "\"", 0, ">", "</a>", 0, IS_URL },
-    { "raw:", 4, 0, 0, 0, 0, 0, DENY_HTML, 0 },
+    { "id:", 3, "<span id=\"", "\"", 0, ">", "</span>", 0, 0 },
+    { "raw:", 4, 0, 0, 0, 0, 0, MKD_NOHTML, 0 },
     { "lang:", 5, "<span lang=\"", "\"", 0, ">", "</span>", 0, 0 },
     { "abbr:", 5, "<abbr title=\"", "\"", 0, ">", "</abbr>", 0, 0 },
     { "class:", 6, "<span class=\"", "\"", 0, ">", "</span>", 0, 0 },
@@ -489,10 +579,13 @@ printlinkyref(MMIOT *f, linkytype *tag, char *link, int size)
 {
     char *edit;
     
+    if ( f->flags & IS_LABEL )
+	return;
+    
     Qstring(tag->link_pfx, f);
 	
     if ( tag->kind & IS_URL ) {
-	if ( f->cb->e_url && (edit = (*f->cb->e_url)(link, size, f->cb->e_data)) ) {
+	if ( f->cb && f->cb->e_url && (edit = (*f->cb->e_url)(link, size, f->cb->e_data)) ) {
 	    puturl(edit, strlen(edit), f, 0);
 	    if ( f->cb->e_free ) (*f->cb->e_free)(edit, f->cb->e_data);
 	}
@@ -500,11 +593,11 @@ printlinkyref(MMIOT *f, linkytype *tag, char *link, int size)
 	    puturl(link + tag->szpat, size - tag->szpat, f, 0);
     }
     else
-	___mkd_reparse(link + tag->szpat, size - tag->szpat, INSIDE_TAG, f);
+	___mkd_reparse(link + tag->szpat, size - tag->szpat, MKD_TAGTEXT, f, 0);
 
     Qstring(tag->link_sfx, f);
 
-    if ( f->cb->e_flags && (edit = (*f->cb->e_flags)(link, size, f->cb->e_data)) ) {
+    if ( f->cb && f->cb->e_flags && (edit = (*f->cb->e_flags)(link, size, f->cb->e_data)) ) {
 	Qchar(' ', f);
 	Qstring(edit, f);
 	if ( f->cb->e_free ) (*f->cb->e_free)(edit, f->cb->e_data);
@@ -512,6 +605,38 @@ printlinkyref(MMIOT *f, linkytype *tag, char *link, int size)
 } /* printlinkyref */
 
 
+/* helper function for php markdown extra footnotes; allow the user to
+ * define a prefix tag instead of just `fn`
+ */
+static char *
+p_or_nothing(p)
+MMIOT *p;
+{
+    return p->ref_prefix ? p->ref_prefix : "fn";
+}
+
+
+/* php markdown extra/daring fireball style print footnotes
+ */
+static int
+extra_linky(MMIOT *f, Cstring text, Footnote *ref)
+{
+    if ( ref->flags & REFERENCED )
+	return 0;
+	
+    if ( f->flags & IS_LABEL )
+    	___mkd_reparse(T(text), S(text), linkt.flags, f, 0);
+    else {
+	ref->flags |= REFERENCED;
+	ref->refnumber = ++ f->footnotes->reference;
+	Qprintf(f, "<sup id=\"%sref:%d\"><a href=\"#%s:%d\" rel=\"footnote\">%d</a></sup>",
+		p_or_nothing(f), ref->refnumber,
+		p_or_nothing(f), ref->refnumber, ref->refnumber);
+    }
+    return 1;
+} /* extra_linky */
+
+
 /* print out a linky (or fail if it's Not Allowed)
  */
 static int
@@ -519,16 +644,17 @@ linkyformat(MMIOT *f, Cstring text, int image, Footnote *ref)
 {
     linkytype *tag;
 
+
     if ( image )
 	tag = &imaget;
     else if ( tag = pseudo(ref->link) ) {
-	if ( f->flags & (NO_PSEUDO_PROTO|SAFELINK) )
+	if ( f->flags & (MKD_NO_EXT|MKD_SAFELINK) )
 	    return 0;
     }
-    else if ( (f->flags & SAFELINK) && T(ref->link)
-				    && (T(ref->link)[0] != '/')
-				    && !isautoprefix(T(ref->link), S(ref->link)) )
-	/* if SAFELINK, only accept links that are local or
+    else if ( (f->flags & MKD_SAFELINK) && T(ref->link)
+				        && (T(ref->link)[0] != '/')
+				        && !isautoprefix(T(ref->link), S(ref->link)) )
+	/* if MKD_SAFELINK, only accept links that are local or
 	 * a well-known protocol
 	 */
 	return 0;
@@ -538,7 +664,9 @@ linkyformat(MMIOT *f, Cstring text, int image, Footnote *ref)
     if ( f->flags & tag->flags )
 	return 0;
 
-    if ( tag->link_pfx ) {
+    if ( f->flags & IS_LABEL )
+	___mkd_reparse(T(text), S(text), tag->flags, f, 0);
+    else if ( tag->link_pfx ) {
 	printlinkyref(f, tag, T(ref->link), S(ref->link));
 
 	if ( tag->WxH ) {
@@ -548,12 +676,12 @@ linkyformat(MMIOT *f, Cstring text, int image, Footnote *ref)
 
 	if ( S(ref->title) ) {
 	    Qstring(" title=\"", f);
-	    ___mkd_reparse(T(ref->title), S(ref->title), INSIDE_TAG, f);
+	    ___mkd_reparse(T(ref->title), S(ref->title), MKD_TAGTEXT, f, 0);
 	    Qchar('"', f);
 	}
 
 	Qstring(tag->text_pfx, f);
-	___mkd_reparse(T(text), S(text), tag->flags, f);
+	___mkd_reparse(T(text), S(text), tag->flags, f, 0);
 	Qstring(tag->text_sfx, f);
     }
     else
@@ -574,6 +702,7 @@ linkylinky(int image, MMIOT *f)
     Footnote key, *ref;
 		
     int status = 0;
+    int extra_footnote = 0;
 
     CREATE(name);
     memset(&key, 0, sizeof key);
@@ -587,7 +716,10 @@ linkylinky(int image, MMIOT *f)
 	else {
 	    int goodlink, implicit_mark = mmiottell(f);
 
-	    if ( eatspace(f) == '[' ) {
+	    if ( isspace(peek(f,1)) )
+		pull(f);
+	    
+	    if ( peek(f,1) == '[' ) {
 		pull(f);	/* consume leading '[' */
 		goodlink = linkylabel(f, &key.tag);
 	    }
@@ -597,6 +729,9 @@ linkylinky(int image, MMIOT *f)
 		 */
 		mmiotseek(f, implicit_mark);
 		goodlink = !(f->flags & MKD_1_COMPAT);
+
+		if ( (f->flags & MKD_EXTRA_FOOTNOTE) && (!image) && S(name) && T(name)[0] == '^' )
+		    extra_footnote = 1;
 	    }
 	    
 	    if ( goodlink ) {
@@ -606,9 +741,14 @@ linkylinky(int image, MMIOT *f)
 		    S(key.tag) = S(name);
 		}
 
-		if ( ref = bsearch(&key, T(*f->footnotes), S(*f->footnotes),
-					  sizeof key, (stfu)__mkd_footsort) )
-		    status = linkyformat(f, name, image, ref);
+		if ( ref = bsearch(&key, T(f->footnotes->note),
+					 S(f->footnotes->note),
+					 sizeof key, (stfu)__mkd_footsort) ) {
+		    if ( extra_footnote )
+			status = extra_linky(f,name,ref);
+		    else
+			status = linkyformat(f, name, image, ref);
+		}
 	    }
 	}
     }
@@ -645,8 +785,12 @@ static void
 mangle(char *s, int len, MMIOT *f)
 {
     while ( len-- > 0 ) {
+#if DEBIAN_GLITCH
+	Qprintf(f, "&#02d;", *((unsigned char*)(s++)) );
+#else
 	Qstring("&#", f);
 	Qprintf(f, COINTOSS() ? "x%02x;" : "%02d;", *((unsigned char*)(s++)) );
+#endif
     }
 }
 
@@ -708,8 +852,10 @@ code(MMIOT *f, char *s, int length)
     int i,c;
 
     for ( i=0; i < length; i++ )
-	if ( (c = s[i]) == 003)  /* ^C: expand back to 2 spaces */
+	if ( (c = s[i]) == MKD_EOLN)  /* ^C: expand back to 2 spaces */
 	    Qstring("  ", f);
+	else if ( c == '\\' && (i < length-1) && escaped(f, s[i+1]) )
+	    cputc(s[++i], f);
 	else
 	    cputc(c, f);
 } /* code */
@@ -721,7 +867,7 @@ static void
 delspan(MMIOT *f, int size)
 {
     Qstring("<del>", f);
-    ___mkd_reparse(cursor(f)-1, size, 0, f);
+    ___mkd_reparse(cursor(f)-1, size, 0, f, 0);
     Qstring("</del>", f);
 }
 
@@ -744,19 +890,19 @@ codespan(MMIOT *f, int size)
 
 
 /* before letting a tag through, validate against
- * DENY_A and DENY_IMG
+ * MKD_NOLINKS and MKD_NOIMAGE
  */
 static int
 forbidden_tag(MMIOT *f)
 {
     int c = toupper(peek(f, 1));
 
-    if ( f->flags & DENY_HTML )
+    if ( f->flags & MKD_NOHTML )
 	return 1;
 
-    if ( c == 'A' && (f->flags & DENY_A) && !isthisalnum(f,2) )
+    if ( c == 'A' && (f->flags & MKD_NOLINKS) && !isthisalnum(f,2) )
 	return 1;
-    if ( c == 'I' && (f->flags & DENY_IMG)
+    if ( c == 'I' && (f->flags & MKD_NOIMAGE)
 		  && strncasecmp(cursor(f)+1, "MG", 2) == 0
 		  && !isthisalnum(f,4) )
 	return 1;
@@ -803,7 +949,7 @@ process_possible_link(MMIOT *f, int size)
     int mailto = 0;
     char *text = cursor(f);
     
-    if ( f->flags & DENY_A ) return 0;
+    if ( f->flags & MKD_NOLINKS ) return 0;
 
     if ( (size > 7) && strncasecmp(text, "mailto:", 7) == 0 ) {
 	/* if it says it's a mailto, it's a mailto -- who am
@@ -850,7 +996,7 @@ maybe_tag_or_link(MMIOT *f)
     int c, size;
     int maybetag = 1;
 
-    if ( f->flags & INSIDE_TAG )
+    if ( f->flags & MKD_TAGTEXT )
 	return 0;
 
     for ( size=0; (c = peek(f, size+1)) != '>'; size++) {
@@ -863,7 +1009,11 @@ maybe_tag_or_link(MMIOT *f)
 	}
 	else if ( isspace(c) )
 	    break;
+#if WITH_GITHUB_TAGS
 	else if ( ! (c == '/' || c == '-' || c == '_' || isalnum(c) ) )
+#else
+	else if ( ! (c == '/' || isalnum(c) ) )
+#endif
 	    maybetag=0;
     }
 
@@ -909,13 +1059,14 @@ maybe_autolink(MMIOT *f)
 
     /* greedily scan forward for the end of a legitimate link.
      */
-    for ( size=0; (c=peek(f, size+1)) != EOF; size++ )
+    for ( size=0; (c=peek(f, size+1)) != EOF; size++ ) {
 	if ( c == '\\' ) {
 	     if ( peek(f, size+2) != EOF )
 		++size;
 	}
-	else if ( isspace(c) || strchr("'\"()[]{}<>`", c) )
+	else if ( isspace(c) || strchr("'\"()[]{}<>`", c) || c == MKD_EOLN )
 	    break;
+    }
 
     if ( (size > 1) && process_possible_link(f, size) ) {
 	shift(f, size);
@@ -954,7 +1105,7 @@ islike(MMIOT *f, char *s)
     int len;
     int i;
 
-    if ( s[0] == '<' ) {
+    if ( s[0] == '|' ) {
 	if ( !isthisnonword(f, -1) )
 	    return 0;
        ++s;
@@ -963,7 +1114,7 @@ islike(MMIOT *f, char *s)
     if ( !(len = strlen(s)) )
 	return 0;
 
-    if ( s[len-1] == '>' ) {
+    if ( s[len-1] == '|' ) {
 	if ( !isthisnonword(f,len-1) )
 	    return 0;
 	len--;
@@ -982,25 +1133,25 @@ static struct smarties {
     char *entity;
     int shift;
 } smarties[] = {
-    { '\'', "'s>",      "rsquo",  0 },
-    { '\'', "'t>",      "rsquo",  0 },
-    { '\'', "'re>",     "rsquo",  0 },
-    { '\'', "'ll>",     "rsquo",  0 },
-    { '\'', "'ve>",     "rsquo",  0 },
-    { '\'', "'m>",      "rsquo",  0 },
-    { '\'', "'d>",      "rsquo",  0 },
-    { '-',  "--",       "mdash",  1 },
-    { '-',  "<->",      "ndash",  0 },
+    { '\'', "'s|",      "rsquo",  0 },
+    { '\'', "'t|",      "rsquo",  0 },
+    { '\'', "'re|",     "rsquo",  0 },
+    { '\'', "'ll|",     "rsquo",  0 },
+    { '\'', "'ve|",     "rsquo",  0 },
+    { '\'', "'m|",      "rsquo",  0 },
+    { '\'', "'d|",      "rsquo",  0 },
+    { '-',  "---",      "mdash",  2 },
+    { '-',  "--",       "ndash",  1 },
     { '.',  "...",      "hellip", 2 },
     { '.',  ". . .",    "hellip", 4 },
     { '(',  "(c)",      "copy",   2 },
     { '(',  "(r)",      "reg",    2 },
     { '(',  "(tm)",     "trade",  3 },
-    { '3',  "<3/4>",    "frac34", 2 },
-    { '3',  "<3/4ths>", "frac34", 2 },
-    { '1',  "<1/2>",    "frac12", 2 },
-    { '1',  "<1/4>",    "frac14", 2 },
-    { '1',  "<1/4th>",  "frac14", 2 },
+    { '3',  "|3/4|",    "frac34", 2 },
+    { '3',  "|3/4ths|", "frac34", 2 },
+    { '1',  "|1/2|",    "frac12", 2 },
+    { '1',  "|1/4|",    "frac14", 2 },
+    { '1',  "|1/4th|",  "frac14", 2 },
     { '&',  "&#0;",      0,       3 },
 } ;
 #define NRSMART ( sizeof smarties / sizeof smarties[0] )
@@ -1013,7 +1164,7 @@ smartypants(int c, int *flags, MMIOT *f)
 {
     int i;
 
-    if ( f->flags & (DENY_SMARTY|INSIDE_TAG) )
+    if ( f->flags & (MKD_NOPANTS|MKD_TAGTEXT|IS_LABEL) )
 	return 0;
 
     for ( i=0; i < NRSMART; i++)
@@ -1042,7 +1193,7 @@ smartypants(int c, int *flags, MMIOT *f)
 			    break;
 			else if ( c == '\'' && peek(f, j+1) == '\'' ) {
 			    Qstring("&ldquo;", f);
-			    ___mkd_reparse(cursor(f)+1, j-2, 0, f);
+			    ___mkd_reparse(cursor(f)+1, j-2, 0, f, 0);
 			    Qstring("&rdquo;", f);
 			    shift(f,j+1);
 			    return 1;
@@ -1062,11 +1213,14 @@ smartypants(int c, int *flags, MMIOT *f)
  * let the caller figure it out.
  */
 static int
-tickhandler(MMIOT *f, int tickchar, int minticks, spanhandler spanner)
+tickhandler(MMIOT *f, int tickchar, int minticks, int allow_space, spanhandler spanner)
 {
     int endticks, size;
     int tick = nrticks(0, tickchar, f);
 
+    if ( !allow_space && isspace(peek(f,tick)) )
+	return 0;
+
     if ( (tick >= minticks) && (size = matchticks(f,tickchar,tick,&endticks)) ) {
 	if ( endticks < tick ) {
 	    size += (tick - endticks);
@@ -1081,7 +1235,7 @@ tickhandler(MMIOT *f, int tickchar, int minticks, spanhandler spanner)
     return 0;
 }
 
-#define tag_text(f)	(f->flags & INSIDE_TAG)
+#define tag_text(f)	(f->flags & MKD_TAGTEXT)
 
 
 static void
@@ -1092,7 +1246,7 @@ text(MMIOT *f)
     int smartyflags = 0;
 
     while (1) {
-        if ( (f->flags & AUTOLINK) && isalpha(peek(f,1)) && !tag_text(f) )
+        if ( (f->flags & MKD_AUTOLINK) && isalpha(peek(f,1)) && !tag_text(f) )
 	    maybe_autolink(f);
 
         c = pull(f);
@@ -1105,7 +1259,8 @@ text(MMIOT *f)
 	switch (c) {
 	case 0:     break;
 
-	case 3:     Qstring(tag_text(f) ? "  " : "<br/>", f);
+	case MKD_EOLN:
+		    Qstring(tag_text(f) ? "  " : "<br/>", f);
 		    break;
 
 	case '>':   if ( tag_text(f) )
@@ -1131,32 +1286,48 @@ text(MMIOT *f)
 	case '[':   if ( tag_text(f) || !linkylinky(0, f) )
 			Qchar(c, f);
 		    break;
-#if SUPERSCRIPT
 	/* A^B -> A<sup>B</sup> */
-	case '^':   if ( (f->flags & (STRICT|INSIDE_TAG)) || isthisspace(f,-1) || isthisspace(f,1) )
+	case '^':   if ( (f->flags & (MKD_NOSUPERSCRIPT|MKD_STRICT|MKD_TAGTEXT))
+				|| (isthisnonword(f,-1) && peek(f,-1) != ')')
+				|| isthisspace(f,1) )
 			Qchar(c,f);
 		    else {
 			char *sup = cursor(f);
 			int len = 0;
-			Qstring("<sup>",f);
-			while ( !isthisspace(f,1+len) ) {
-			    ++len;
+
+			if ( peek(f,1) == '(' ) {
+			    int here = mmiottell(f);
+			    pull(f);
+
+			    if ( (len = parenthetical('(',')',f)) <= 0 ) {
+				mmiotseek(f,here);
+				Qchar(c, f);
+				break;
+			    }
+			    sup++;
 			}
-			shift(f,len);
-			___mkd_reparse(sup, len, 0, f);
+			else {
+			    while ( isthisalnum(f,1+len) )
+				++len;
+			    if ( !len ) {
+				Qchar(c,f);
+				break;
+			    }
+			    shift(f,len);
+			}
+			Qstring("<sup>",f);
+			___mkd_reparse(sup, len, 0, f, "()");
 			Qstring("</sup>", f);
 		    }
 		    break;
-#endif
 	case '_':
-#if RELAXED_EMPHASIS
 	/* Underscores don't count if they're in the middle of a word */
-		    if ( !(f->flags & STRICT) && isthisalnum(f,-1)
-					      && isthisalnum(f,1) ) {
+		    if ( !(f->flags & (MKD_NORELAXED|MKD_STRICT))
+					&& isthisalnum(f,-1)
+					 && isthisalnum(f,1) ) {
 			Qchar(c, f);
 			break;
 		    }
-#endif
 	case '*':
 	/* Underscores & stars don't count if they're out in the middle
 	 * of whitespace */
@@ -1174,29 +1345,55 @@ text(MMIOT *f)
 		    }
 		    break;
 	
-	case '~':   if ( (f->flags & (NOSTRIKETHROUGH|INSIDE_TAG|STRICT)) || !tickhandler(f,c,2,delspan) )
+	case '~':   if ( (f->flags & (MKD_NOSTRIKETHROUGH|MKD_TAGTEXT|MKD_STRICT)) || ! tickhandler(f,c,2,0, delspan) )
 			Qchar(c, f);
 		    break;
 
-	case '`':   if ( tag_text(f) || !tickhandler(f,c,1,codespan) )
+	case '`':   if ( tag_text(f) || !tickhandler(f,c,1,1,codespan) )
 			Qchar(c, f);
 		    break;
 
 	case '\\':  switch ( c = pull(f) ) {
 		    case '&':   Qstring("&amp;", f);
 				break;
-		    case '<':   Qstring("&lt;", f);
+		    case '<':   c = peek(f,1);
+				if ( (c == EOF) || isspace(c) )
+				    Qstring("&lt;", f);
+				else {
+				    /* Markdown.pl does not escape <[nonwhite]
+				     * sequences */
+				    Qchar('\\', f);
+				    shift(f, -1);
+				}
+				
 				break;
-		    case '>': case '#': case '.': case '-':
-		    case '+': case '{': case '}': case ']':
-		    case '!': case '[': case '*': case '_':
-		    case '\\':case '(': case ')':
-		    case '`':	Qchar(c, f);
-				break;
-		    default:
-				Qchar('\\', f);
-				if ( c != EOF )
+		    case '^':   if ( f->flags & (MKD_STRICT|MKD_NOSUPERSCRIPT) ) {
+				    Qchar('\\', f);
 				    shift(f,-1);
+				    break;
+				}
+				Qchar(c, f);
+				break;
+				
+		    case ':': case '|':
+				if ( f->flags & MKD_NOTABLES ) {
+				    Qchar('\\', f);
+				    shift(f,-1);
+				    break;
+				}
+				Qchar(c, f);
+				break;
+				
+		    case EOF:	Qchar('\\', f);
+				break;
+				
+		    default:    if ( escaped(f,c) ||
+				     strchr(">#.-+{}]![*_\\()`", c) )
+				    Qchar(c, f);
+				else {
+				    Qchar('\\', f);
+				    shift(f, -1);
+				}
 				break;
 		    }
 		    break;
@@ -1229,13 +1426,26 @@ text(MMIOT *f)
 static void
 printheader(Paragraph *pp, MMIOT *f)
 {
+#if WITH_ID_ANCHOR
     Qprintf(f, "<h%d", pp->hnumber);
-    if ( f->flags & TOC ) {
-	Qprintf(f, " id=\"", pp->hnumber);
-	mkd_string_to_anchor(T(pp->text->text), S(pp->text->text), Qchar, f);
+    if ( f->flags & MKD_TOC ) {
+	Qstring(" id=\"", f);
+	mkd_string_to_anchor(T(pp->text->text),
+			     S(pp->text->text),
+			     (mkd_sta_function_t)Qchar, f, 1);
 	Qchar('"', f);
     }
     Qchar('>', f);
+#else
+    if ( f->flags & MKD_TOC ) {
+	Qstring("<a name=\"", f);
+	mkd_string_to_anchor(T(pp->text->text),
+			     S(pp->text->text),
+			     (mkd_sta_function_t)Qchar, f, 1);
+	Qstring("\"></a>\n", f);
+    }
+    Qprintf(f, "<h%d>", pp->hnumber);
+#endif
     push(T(pp->text->text), S(pp->text->text), f);
     text(f);
     Qprintf(f, "</h%d>", pp->hnumber);
@@ -1244,8 +1454,9 @@ printheader(Paragraph *pp, MMIOT *f)
 
 enum e_alignments { a_NONE, a_CENTER, a_LEFT, a_RIGHT };
 
-static char* alignments[] = { "", " align=\"center\"", " align=\"left\"",
-				  " align=\"right\"" };
+static char* alignments[] = { "", " style=\"text-align:center;\"",
+				  " style=\"text-align:left;\"",
+				  " style=\"text-align:right;\"" };
 
 typedef STRING(int) Istring;
 
@@ -1253,22 +1464,30 @@ static int
 splat(Line *p, char *block, Istring align, int force, MMIOT *f)
 {
     int first,
-	idx = 0,
+	idx = p->dle,
 	colno = 0;
 
+
+    ___mkd_tidy(&p->text);
+    if ( T(p->text)[S(p->text)-1] == '|' )
+	--S(p->text);
+    
     Qstring("<tr>\n", f);
     while ( idx < S(p->text) ) {
 	first = idx;
 	if ( force && (colno >= S(align)-1) )
 	    idx = S(p->text);
 	else
-	    while ( (idx < S(p->text)) && (T(p->text)[idx] != '|') )
+	    while ( (idx < S(p->text)) && (T(p->text)[idx] != '|') ) {
+		if ( T(p->text)[idx] == '\\' )
+		    ++idx;
 		++idx;
+	    }
 
 	Qprintf(f, "<%s%s>",
 		   block,
 		   alignments[ (colno < S(align)) ? T(align)[colno] : a_NONE ]);
-	___mkd_reparse(T(p->text)+first, idx-first, 0, f);
+	___mkd_reparse(T(p->text)+first, idx-first, 0, f, "|");
 	Qprintf(f, "</%s>\n", block);
 	idx++;
 	colno++;
@@ -1290,34 +1509,43 @@ printtable(Paragraph *pp, MMIOT *f)
 
     Line *hdr, *dash, *body;
     Istring align;
-    int start;
-    int hcols;
+    int hcols,start;
     char *p;
-
-    if ( !(pp->text && pp->text->next) )
-	return 0;
+    enum e_alignments it;
 
     hdr = pp->text;
     dash= hdr->next;
     body= dash->next;
 
-    /* first figure out cell alignments */
+    if ( T(hdr->text)[hdr->dle] == '|' ) {
+	/* trim leading pipe off all lines
+	 */
+	Line *r;
+	for ( r = pp->text; r; r = r->next )
+	    r->dle ++;
+    }
+
+    /* figure out cell alignments */
 
     CREATE(align);
 
-    for (p=T(dash->text), start=0; start < S(dash->text); ) {
+    for (p=T(dash->text), start=dash->dle; start < S(dash->text); ) {
 	char first, last;
 	int end;
 	
 	last=first=0;
 	for (end=start ; (end < S(dash->text)) && p[end] != '|'; ++ end ) {
-	    if ( !isspace(p[end]) ) {
+	    if ( p[end] == '\\' )
+		++ end;
+	    else if ( !isspace(p[end]) ) {
 		if ( !first) first = p[end];
 		last = p[end];
 	    }
 	}
-	EXPAND(align) = ( first == ':' ) ? (( last == ':') ? a_CENTER : a_LEFT)
-					 : (( last == ':') ? a_RIGHT : a_NONE );
+	it = ( first == ':' ) ? (( last == ':') ? a_CENTER : a_LEFT)
+			      : (( last == ':') ? a_RIGHT : a_NONE );
+
+	EXPAND(align) = it;
 	start = 1+end;
     }
 
@@ -1356,13 +1584,14 @@ printblock(Paragraph *pp, MMIOT *f)
 			 && T(t->text)[S(t->text)-2] == ' '
 			 && T(t->text)[S(t->text)-1] == ' ' ) {
 		push(T(t->text), S(t->text)-2, f);
-		push("\003\n", 2, f);
+		pushc(MKD_EOLN, f);
+		pushc('\n', f);
 	    }
 	    else {
 		___mkd_tidy(&t->text);
 		push(T(t->text), S(t->text), f);
 		if ( t->next )
-		    push("\n", 1, f);
+		    pushc('\n', f);
 	    }
 	}
 	t = t->next;
@@ -1375,11 +1604,17 @@ printblock(Paragraph *pp, MMIOT *f)
 
 
 static void
-printcode(Line *t, MMIOT *f)
+printcode(Line *t, char *lang, MMIOT *f)
 {
     int blanks;
 
-    Qstring("<pre><code>", f);
+    Qstring("<pre><code", f);
+    if (lang) {
+      Qstring(" class=\"", f);
+      Qstring(lang, f);
+      Qstring("\"", f);
+    }
+    Qstring(">", f);
     for ( blanks = 0; t ; t = t->next ) {
 	if ( S(t->text) > t->dle ) {
 	    while ( blanks ) {
@@ -1432,7 +1667,6 @@ htmlify(Paragraph *p, char *block, char *arguments, MMIOT *f)
 }
 
 
-#if DL_TAG_EXTENSION
 static void
 definitionlist(Paragraph *p, MMIOT *f)
 {
@@ -1444,7 +1678,7 @@ definitionlist(Paragraph *p, MMIOT *f)
 	for ( ; p ; p = p->next) {
 	    for ( tag = p->text; tag; tag = tag->next ) {
 		Qstring("<dt>", f);
-		___mkd_reparse(T(tag->text), S(tag->text), 0, f);
+		___mkd_reparse(T(tag->text), S(tag->text), 0, f, 0);
 		Qstring("</dt>\n", f);
 	    }
 
@@ -1455,7 +1689,6 @@ definitionlist(Paragraph *p, MMIOT *f)
 	Qstring("</dl>", f);
     }
 }
-#endif
 
 
 static void
@@ -1464,7 +1697,7 @@ listdisplay(int typ, Paragraph *p, MMIOT* f)
     if ( p ) {
 	Qprintf(f, "<%cl", (typ==UL)?'u':'o');
 	if ( typ == AL )
-	    Qprintf(f, " type=a");
+	    Qprintf(f, " type=\"a\"");
 	Qprintf(f, ">\n");
 
 	for ( ; p ; p = p->next ) {
@@ -1494,7 +1727,7 @@ display(Paragraph *p, MMIOT *f)
 	break;
 	
     case CODE:
-	printcode(p->text, f);
+	printcode(p->text, p->lang, f);
 	break;
 	
     case QUOTE:
@@ -1507,11 +1740,9 @@ display(Paragraph *p, MMIOT *f)
 	listdisplay(p->typ, p->down, f);
 	break;
 
-#if DL_TAG_EXTENSION
     case DL:
 	definitionlist(p->down, f);
 	break;
-#endif
 
     case HR:
 	Qstring("<hr />", f);
@@ -1537,20 +1768,59 @@ display(Paragraph *p, MMIOT *f)
 }
 
 
+/* dump out a list of footnotes
+ */
+static void
+mkd_extra_footnotes(MMIOT *m)
+{
+    int j, i;
+    Footnote *t;
+
+    if ( m->footnotes->reference == 0 )
+	return;
+
+    Csprintf(&m->out, "\n<div class=\"footnotes\">\n<hr/>\n<ol>\n");
+    
+    for ( i=1; i <= m->footnotes->reference; i++ ) {
+	for ( j=0; j < S(m->footnotes->note); j++ ) {
+	    t = &T(m->footnotes->note)[j];
+	    if ( (t->refnumber == i) && (t->flags & REFERENCED) ) {
+		Csprintf(&m->out, "<li id=\"%s:%d\">\n<p>",
+			    p_or_nothing(m), t->refnumber);
+		Csreparse(&m->out, T(t->title), S(t->title), 0);
+		Csprintf(&m->out, "<a href=\"#%sref:%d\" rev=\"footnote\">&#8617;</a>",
+			    p_or_nothing(m), t->refnumber);
+		Csprintf(&m->out, "</p></li>\n");
+	    }
+	}
+    }
+    Csprintf(&m->out, "</ol>\n</div>\n");
+}
+
+
 /* return a pointer to the compiled markdown
  * document.
  */
 int
 mkd_document(Document *p, char **res)
 {
+    int size;
+    
     if ( p && p->compiled ) {
 	if ( ! p->html ) {
 	    htmlify(p->code, 0, 0, p->ctx);
+	    if ( p->ctx->flags & MKD_EXTRA_FOOTNOTE )
+		mkd_extra_footnotes(p->ctx);
 	    p->html = 1;
 	}
 
+	size = S(p->ctx->out);
+	
+	if ( (size == 0) || T(p->ctx->out)[size-1] )
+	    EXPAND(p->ctx->out) = 0;
+	
 	*res = T(p->ctx->out);
-	return S(p->ctx->out);
+	return size;
     }
     return EOF;
 }
